==> tensorstore-0.1.53.tar.gz: tensorstore-0.1.53/tensorstore/internal/http/http_response.h <==
// Copyright 2020 The TensorStore Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef TENSORSTORE_INTERNAL_HTTP_HTTP_RESPONSE_H_
#define TENSORSTORE_INTERNAL_HTTP_HTTP_RESPONSE_H_

#include <stddef.h>
#include <stdint.h>

#include <map>
#include <optional>
#include <string>
#include <string_view>
#include <tuple>

#include "absl/status/status.h"
#include "absl/strings/cord.h"
#include "absl/strings/numbers.h"
#include "absl/strings/str_format.h"
#include "tensorstore/internal/source_location.h"
#include "tensorstore/util/result.h"

namespace tensorstore {
namespace internal_http {

/// AppendHeaderData parses `data` as a header and append to the set of
/// `headers`.
size_t AppendHeaderData(std::multimap<std::string, std::string>& headers,
                        std::string_view data);

/// HttpResponse contains the results of an HTTP request.
struct HttpResponse {
  int32_t status_code;
  absl::Cord payload;
  std::multimap<std::string, std::string> headers;

  template <typename Sink>
  friend void AbslStringify(Sink& sink, const HttpResponse& response) {
    absl::Format(&sink, "HttpResponse{code=%d, headers=<",
                 response.status_code);
    const char* sep = "";
    for (const auto& kv : response.headers) {
      sink.Append(sep);
      sink.Append(kv.first);
      sink.Append("=");
      sink.Append(kv.second);
      sep = ", ";
    }
    absl::Format(&sink, ">, body=%v}", response.payload);
  }
};

/// Returns an `absl::Status` object for a corresponding
/// HttpResponse.status_code.
absl::Status HttpResponseCodeToStatus(
    const HttpResponse& response,
    SourceLocation loc = ::tensorstore::SourceLocation::current());

/// Parses the "content-range" header, which can be used to determine the
/// portion of an object returned by an HTTP request (with status code 206).
/// Returned tuple fields are {start, end, total_length}
Result<std::tuple<size_t, size_t, size_t>> ParseContentRangeHeader(
    const HttpResponse& response);

/// `strptime`-compatible format string for the HTTP date header.
///
/// https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Date
///
/// Note that the time zone is always UTC and is specified as "GMT".
constexpr const char kHttpTimeFormat[] = "%a, %d %b %E4Y %H:%M:%S GMT";

/// Attempts to parse a header using SimpleAtoi.
template <typename T>
std::optional<T> TryParseIntHeader(
    const std::multimap<std::string, std::string>& headers,
    const std::string& header) {
  auto it = headers.find(header);
  T result;
  if (it != headers.end() && absl::SimpleAtoi(it->second, &result)) {
    return result;
  }
  return std::nullopt;
}

/// Attempts to parse a header using SimpleAtob.
std::optional<bool> TryParseBoolHeader(
    const std::multimap<std::string, std::string>& headers,
    const std::string& header);

}  // namespace internal_http
}  // namespace tensorstore

#endif  // TENSORSTORE_INTERNAL_HTTP_HTTP_RESPONSE_H_

==> tensorstore-0.1.56.tar.gz: tensorstore-0.1.56/tensorstore/internal/http/http_response.h <==
// Copyright 2020 The TensorStore Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef TENSORSTORE_INTERNAL_HTTP_HTTP_RESPONSE_H_
#define TENSORSTORE_INTERNAL_HTTP_HTTP_RESPONSE_H_

#include <stddef.h>
#include <stdint.h>

#include <map>
#include <optional>
#include <string>
#include <string_view>
#include <tuple>

#include "absl/status/status.h"
#include "absl/strings/cord.h"
#include "absl/strings/match.h"
#include "absl/strings/numbers.h"
#include "absl/strings/str_format.h"
#include "tensorstore/internal/source_location.h"
#include "tensorstore/util/result.h"

namespace tensorstore {
namespace internal_http {

/// AppendHeaderData parses `data` as a header and append to the set of
/// `headers`.
size_t AppendHeaderData(std::multimap<std::string, std::string>& headers,
                        std::string_view data);

/// HttpResponse contains the results of an HTTP request.
struct HttpResponse {
  int32_t status_code;
  absl::Cord payload;
  std::multimap<std::string, std::string> headers;

  template <typename Sink>
  friend void AbslStringify(Sink& sink, const HttpResponse& response) {
    absl::Format(&sink, "HttpResponse{code=%d, headers=<",
                 response.status_code);
    const char* sep = "";
    for (const auto& kv : response.headers) {
      sink.Append(sep);
      sink.Append(kv.first);
      sink.Append(": ");
#ifndef NDEBUG
      // Redact auth_token in response logging.
      if (absl::StrContainsIgnoreCase(kv.first, "auth_token")) {
        sink.Append("#####");
      } else
#endif
      {
        sink.Append(kv.second);
      }
      sep = ", ";
    }
    if (response.status_code >= 300 && response.payload.size() <= 64) {
      absl::Format(&sink, ">, payload=%v}", response.payload);
    } else {
      absl::Format(&sink, ">, payload.size=%d}", response.payload.size());
    }
  }
};

/// Returns an `absl::Status` object for a corresponding
/// HttpResponse.status_code.
absl::Status HttpResponseCodeToStatus(
    const HttpResponse& response,
    SourceLocation loc = ::tensorstore::SourceLocation::current());

/// Parses the "content-range" header, which can be used to determine the
/// portion of an object returned by an HTTP request (with status code 206).
/// Returned tuple fields are {start, end, total_length}
Result<std::tuple<size_t, size_t, size_t>> ParseContentRangeHeader(
    const HttpResponse& response);

/// `strptime`-compatible format string for the HTTP date header.
///
/// https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Date
///
/// Note that the time zone is always UTC and is specified as "GMT".
constexpr const char kHttpTimeFormat[] = "%a, %d %b %E4Y %H:%M:%S GMT";

/// Attempts to parse a header using SimpleAtoi.
template <typename T>
std::optional<T> TryParseIntHeader(
    const std::multimap<std::string, std::string>& headers,
    const std::string& header) {
  auto it = headers.find(header);
  T result;
  if (it != headers.end() && absl::SimpleAtoi(it->second, &result)) {
    return result;
  }
  return std::nullopt;
}

/// Attempts to parse a header using SimpleAtob.
std::optional<bool> TryParseBoolHeader(
    const std::multimap<std::string, std::string>& headers,
    const std::string& header);

}  // namespace internal_http
}  // namespace tensorstore

#endif  // TENSORSTORE_INTERNAL_HTTP_HTTP_RESPONSE_H_
